@@ -288,7 +288,7 @@ bool WindowsPoller::Poll(TimeoutManager *timeout_manager,
                          const TimeInterval &poll_interval) {
   TimeInterval sleep_interval = poll_interval;
   TimeStamp now;
-  m_clock->CurrentTime(&now);
+  m_clock->CurrentMonotonicTime(&now);
 
   TimeInterval next_event_in = timeout_manager->ExecuteTimeouts(&now);
   if (!next_event_in.IsZero()) {
@@ -473,7 +473,7 @@ bool WindowsPoller::Poll(TimeoutManager *timeout_manager,
     CancelIOs(&data);
 
     if (result == WAIT_TIMEOUT) {
-      m_clock->CurrentTime(&m_wake_up_time);
+      m_clock->CurrentMonotonicTime(&m_wake_up_time);
       timeout_manager->ExecuteTimeouts(&m_wake_up_time);
       // We can't return here since any of the cancelled IO calls still might
       // have succeeded.
@@ -509,7 +509,7 @@ bool WindowsPoller::Poll(TimeoutManager *timeout_manager,
     CancelIOs(&data);
   }
 
-  m_clock->CurrentTime(&m_wake_up_time);
+  m_clock->CurrentMonotonicTime(&m_wake_up_time);
   timeout_manager->ExecuteTimeouts(&m_wake_up_time);
 
   FinalCheckIOs(data);
@@ -535,7 +535,7 @@ bool WindowsPoller::Poll(TimeoutManager *timeout_manager,
   STLDeleteElements(&data);
   STLDeleteElements(&event_holders);
 
-  m_clock->CurrentTime(&m_wake_up_time);
+  m_clock->CurrentMonotonicTime(&m_wake_up_time);
   timeout_manager->ExecuteTimeouts(&m_wake_up_time);
   return return_value;
 }
