@@ -326,6 +326,11 @@ typedef enum {
 } rdm_data_type;
 
 
+// Remember to update the other places when adding new NACK codes:
+// common/rdm/RDMHelper.cpp
+// plugins/usbpro/DmxTriWidget.cpp
+// plugins/usbpro/DmxTriWidget.h
+// python/ola/OlaClient.py
 typedef enum {
   NR_UNKNOWN_PID = 0x0000,
   NR_FORMAT_ERROR = 0x0001,
@@ -339,7 +344,14 @@ typedef enum {
   NR_SUB_DEVICE_OUT_OF_RANGE = 0x0009,
   NR_PROXY_BUFFER_FULL = 0x000A,
   NR_ACTION_NOT_SUPPORTED = 0x000B,
-  NR_ENDPOINT_NUMBER_INVALID = 0x0011
+  NR_ENDPOINT_NUMBER_INVALID = 0x000C,
+  NR_INVALID_ENDPOINT_MODE = 0x000D,
+  NR_UNKNOWN_UID = 0x000E,
+  NR_UNKNOWN_SCOPE = 0x000F,
+  NR_INVALID_STATIC_CONFIG_TYPE = 0x0010,
+  NR_INVALID_IPV4_ADDRESS = 0x0011,
+  NR_INVALID_IPV6_ADDRESS = 0x0012,
+  NR_INVALID_PORT = 0x0013
 } rdm_nack_reason;
 
 
@@ -692,6 +704,9 @@ typedef enum {
 static const uint16_t IPV4_UNCONFIGURED = 0x00000000;
 static const uint16_t NO_DEFAULT_ROUTE = 0x00000000;
 
+static const uint32_t MIN_RDM_INTERFACE_INDEX = 0x00000001;
+static const uint32_t MAX_RDM_INTERFACE_INDEX = 0xFFFFFF00;
+
 typedef enum {
   DHCP_STATUS_INACTIVE = 0x00,
   DHCP_STATUS_ACTIVE = 0x01,
